@@ -5,14 +5,9 @@ import plotly.figure_factory as ff
 import plotly.graph_objects as go
 from tqdm.auto import tqdm
 
-# from src.pitchly.params import FIELD_COLOR
-# from src.pitchly.params import FIELD_DIM
-# from src.pitchly.params import FIELD_HEIGHT
-# from src.pitchly.params import FIELD_MARKINGS_COLOR
-# from src.pitchly.params import FIELD_WIDTH
-from src.pitchly.params import event_player_marker_args
-from src.pitchly.params import player_marker_args
+from src.pitchly.params import prm
 from src.pitchly.pitch import Pitch
+from src.pitchly.pitch_control_old import generate_pitch_control_for_frame
 
 
 class TrackingData:
@@ -20,14 +15,6 @@ class TrackingData:
         self.tracking_home = tracking_home
         self.tracking_away = tracking_away
 
-    def get_frame_data(self, frameID):
-
-        hometeam = self.tracking_home.loc[frameID]
-        awayteam = self.tracking_away.loc[frameID]
-        frame_data = (hometeam, awayteam)
-
-        return frame_data
-
     def get_mins(self, time):
         if ":" not in time:
             return f"{time}:0.04"
@@ -37,17 +24,69 @@ class TrackingData:
     def get_frameID_from_mins(self, mins):
         return self.tracking_home.query("mins==@mins").index[0]
 
+    def get_frame_data(self, frameID):
+
+        hometeam = self.tracking_home.loc[frameID]
+        awayteam = self.tracking_away.loc[frameID]
+        frame_data = (hometeam, awayteam)
+
+        return frame_data
+
+    def get_team_pitch_control_traces(self, frame_data, player_num=None):
+        pitch_control_dict = generate_pitch_control_for_frame(frame_data)
+        if player_num:
+            surface = pitch_control_dict["PPCFa_pax"][str(player_num)]
+        else:
+            surface = pitch_control_dict["PPCFa"]
+
+        trace = go.Heatmap(
+            z=surface,
+            x=pitch_control_dict["xgrid"],
+            y=pitch_control_dict["ygrid"],
+            colorscale="RdBu_r",
+            opacity=0.8,
+            zsmooth="best",
+            zmin=0.0,
+            zmax=1.0,
+            # showlegend=False,
+            colorbar={"len": 0.3, "thickness": 10, "x": 0.9},
+            showscale=False,
+        )
+        # trace = go.Surface(
+        # z=surface,
+        # colorscale='RdBu_r', opacity=0.8,
+        # cmin=0, cmax=1
+        # )
+        return [trace]
+
     def position_traces(self, frame_data):
 
         position_traces = []
         for i, side in enumerate(["Home", "Away"]):
             team_data = frame_data[i]
             player_nums = list(
-                set(item for subitem in team_data.keys() for item in subitem.split("_") if item.isdigit())
+                set(
+                    item
+                    for subitem in team_data.keys()
+                    for item in subitem.split("_")
+                    if item.isdigit()
+                )
+            )
+            xlocs = [
+                team_data["{}_{}_{}".format(side, num, "X")]
+                for num in player_nums
+            ]
+            ylocs = [
+                team_data["{}_{}_{}".format(side, num, "Y")]
+                for num in player_nums
+            ]
+            traces = go.Scatter(
+                x=xlocs,
+                y=ylocs,
+                text=player_nums,
+                **prm.player_marker_args[side],
+                name=side,
             )
-            xlocs = [team_data["{}_{}_{}".format(side, num, "X")] for num in player_nums]
-            ylocs = [team_data["{}_{}_{}".format(side, num, "Y")] for num in player_nums]
-            traces = go.Scatter(x=xlocs, y=ylocs, text=player_nums, **player_marker_args[side], name=side)
             position_traces.append(traces)
 
         return position_traces
@@ -58,19 +97,36 @@ class TrackingData:
         for i, side in enumerate(["Home", "Away"]):
             team_data = frame_data[i]
             player_nums = list(
-                set(item for subitem in team_data.keys() for item in subitem.split("_") if item.isdigit())
+                set(
+                    item
+                    for subitem in team_data.keys()
+                    for item in subitem.split("_")
+                    if item.isdigit()
+                )
             )
-            xlocs = [team_data["{}_{}_{}".format(side, num, "X")] for num in player_nums]
-            ylocs = [team_data["{}_{}_{}".format(side, num, "Y")] for num in player_nums]
-            xvels = [team_data["{}_{}_{}".format(side, num, "vx")] for num in player_nums]
-            yvels = [team_data["{}_{}_{}".format(side, num, "vy")] for num in player_nums]
+            xlocs = [
+                team_data["{}_{}_{}".format(side, num, "X")]
+                for num in player_nums
+            ]
+            ylocs = [
+                team_data["{}_{}_{}".format(side, num, "Y")]
+                for num in player_nums
+            ]
+            xvels = [
+                team_data["{}_{}_{}".format(side, num, "vx")]
+                for num in player_nums
+            ]
+            yvels = [
+                team_data["{}_{}_{}".format(side, num, "vy")]
+                for num in player_nums
+            ]
             trace = ff.create_quiver(
                 x=xlocs,
                 y=ylocs,
                 u=xvels,
                 v=yvels,
                 scale=0.5,
-                line_color=player_marker_args[side]["marker_color"],
+                line_color=prm.player_marker_args[side]["marker_color"],
                 name=side + "_vel",
             )
             velocity_quivers.append(trace.data[0])
@@ -85,15 +141,22 @@ class TrackingData:
             y=[team_data["ball_Y"]],
             marker_size=10,
             marker_opacity=0.8,
-            marker_color="black",
+            marker_color="white",
             marker_line_width=2,
-            marker_line_color="green",
+            marker_line_color="black",
             name="ball",
         )
 
         return ball_trace
 
-    def get_traces(self, frameID=None, velocities=True, ball=True):
+    def get_traces(
+        self,
+        frameID=None,
+        pitch_control=False,
+        velocities=True,
+        ball=True,
+        player_num=None,
+    ):
         """Combines various traces for required plot and returns it
 
         Args:
@@ -103,6 +166,13 @@ class TrackingData:
         frame_data = self.get_frame_data(frameID)
 
         traces = []
+
+        if pitch_control:
+            traces.extend(
+                self.get_team_pitch_control_traces(
+                    frame_data, player_num=player_num
+                )
+            )
         if velocities:
             traces.extend(self.velocity_traces(frame_data))
 
@@ -113,17 +183,26 @@ class TrackingData:
 
         return traces
 
-    def get_frames(self, frame_range, velocities=True, ball=True):
+    def get_frames(
+        self, frame_range, pitch_control=False, velocities=True, ball=True
+    ):
 
         frames = []
         for frameID in tqdm(frame_range):
-            frame = {"data": [], "name": str(frameID)}
-            frame["data"].extend(self.get_traces(frameID, velocities, ball))
-            frames.append(frame)
+            data_ = self.get_traces(frameID, pitch_control, velocities, ball)
+            name_ = f"f{frameID}"
+            frames.append(go.Frame(data=data_, name=name_))
 
         return frames
 
-    def plot_frame(self, frameID=None, time=None, plot_ball=True, show_velocities=False):
+    def plot_frame(
+        self,
+        frameID=None,
+        time=None,
+        pitch_control=False,
+        plot_ball=True,
+        show_velocities=False,
+    ):
 
         if time:
             time = self.get_mins(time)
@@ -132,11 +211,25 @@ class TrackingData:
             time = self.tracking_home.loc[frameID, "mins"]
 
         title = f"Time: [{time}] | FrameID: {frameID}"
-        data = self.get_traces(frameID=frameID, velocities=show_velocities, ball=plot_ball)
+        data = self.get_traces(
+            frameID=frameID,
+            pitch_control=pitch_control,
+            velocities=show_velocities,
+            ball=plot_ball,
+        )
         pitch = Pitch()
-        return pitch.plot_freeze_frame(data, title)
+        return pitch.plot_freeze_frame(data, title, pitch_control)
 
-    def plot_sequence(self, f0=None, f1=None, t0=None, t1=None, show_velocities=True, player_num=None):
+    def plot_sequence(
+        self,
+        f0=None,
+        f1=None,
+        t0=None,
+        t1=None,
+        pitch_control=False,
+        show_velocities=True,
+        player_num=None,
+    ):
 
         if t1:
             t0 = self.get_mins(t0)
@@ -150,10 +243,20 @@ class TrackingData:
         frame_range = range(f0, f1)
 
         title = f"Time: [{t0}] | FrameID: {f0} to {f1}"
-        data = self.get_traces(frameID=f0, velocities=show_velocities)
-        frames = self.get_frames(frame_range, velocities=show_velocities)
+        data = self.get_traces(
+            frameID=f0, pitch_control=pitch_control, velocities=show_velocities
+        )
+        frames = self.get_frames(
+            frame_range, pitch_control=pitch_control, velocities=show_velocities
+        )
         pitch = Pitch()
-        return pitch.plot_frames_sequence(data, frames, frame_range, title)
+        return pitch.plot_frames_sequence(
+            data,
+            frames,
+            frame_range,
+            title,
+            pitch_control
+        )
 
 
 class EventData:
@@ -172,7 +275,9 @@ class EventData:
             if not isinstance(self.events.loc[i, "Subtype"], str):
                 continue
             # if self.events["Type"].iloc[i] == "SHOT":
-            if self.events["Subtype"].iloc[i].endswith("SAVED") or self.events["Subtype"].iloc[i].endswith("BLOCKED"):
+            if self.events["Subtype"].iloc[i].endswith("SAVED") or self.events[
+                "Subtype"
+            ].iloc[i].endswith("BLOCKED"):
                 self.events.loc[i, "marker_color"] = "white"
                 self.events.loc[i, "marker_line_color"] = "white"
                 self.events.loc[i, "marker_line_width"] = 2
@@ -260,20 +365,20 @@ class EventData:
             if row["Type"] != "PASS":
                 break
         buildup = self.events.iloc[i + 1 : index + 1, :]
-        buildup["From_num"] = buildup.From.apply(lambda x: re.findall("\d+", str(x))).apply(
-            lambda x: x[0] if len(x) > 0 else None
-        )
-        buildup["To_num"] = buildup.To.apply(lambda x: re.findall(r"\d+", str(x))).apply(
-            lambda x: x[0] if len(x) > 0 else None
-        )
+        buildup["From_num"] = buildup.From.apply(
+            lambda x: re.findall("\d+", str(x))
+        ).apply(lambda x: x[0] if len(x) > 0 else None)
+        buildup["To_num"] = buildup.To.apply(
+            lambda x: re.findall(r"\d+", str(x))
+        ).apply(lambda x: x[0] if len(x) > 0 else None)
         buildup["To_prev"] = buildup.To_num.shift(1)
         buildup["Start X Carry"] = buildup["End X"]
         buildup["Start Y Carry"] = buildup["End Y"]
         buildup["End X Carry"] = buildup["Start X"].shift(-1)
         buildup["End Y Carry"] = buildup["Start Y"].shift(-1)
-        buildup["carry"] = (buildup["Start X Carry"] != buildup["End X Carry"]) | (
-            buildup["Start Y Carry"] != buildup["End Y Carry"]
-        )
+        buildup["carry"] = (
+            buildup["Start X Carry"] != buildup["End X Carry"]
+        ) | (buildup["Start Y Carry"] != buildup["End Y Carry"])
         buildup.loc[buildup["carry"] == False, "Start X Carry"] = np.nan
         buildup.loc[buildup["carry"] == False, "Start Y Carry"] = np.nan
         buildup.loc[buildup["carry"] == False, "End X Carry"] = np.nan
@@ -292,7 +397,15 @@ class EventData:
         side = row["Team"]
         player_nums.extend([row["From_num"], row["To_num"]])
 
-        event_traces.append(go.Scatter(x=x, y=y, text=player_nums, name=row["Type"], **event_player_marker_args[side]))
+        event_traces.append(
+            go.Scatter(
+                x=x,
+                y=y,
+                text=player_nums,
+                name=row["Type"],
+                **prm.event_player_marker_args[side],
+            )
+        )
 
         if (row["Type"] == "PASS") & (row["carry"] == True):
             event_traces.append(
@@ -302,7 +415,7 @@ class EventData:
                     text=[row["To_num"]],
                     name="DRIBBLE",
                     line_dash="dash",
-                    **event_player_marker_args[side],
+                    **prm.event_player_marker_args[side],
                 )
             )
 
@@ -320,7 +433,9 @@ class EventData:
 
         if index:
             pitch = Pitch()
-            return pitch.plot_event(data=self.get_traces(index), title=self.title(index))
+            return pitch.plot_event(
+                data=self.get_traces(index), title=self.title(index)
+            )
 
         if type == "shots":
             data = self.get_shots()
@@ -381,10 +496,14 @@ class EventData:
                         mode="lines+markers+text" if trace else "markers+text",
                         marker_size=[15, 0],
                         marker_symbol="circle-x",
-                        marker_color="#AD0B05" if row["Team"] == "Home" else "#0570B0",
+                        marker_color="#AD0B05"
+                        if row["Team"] == "Home"
+                        else "#0570B0",
                         marker_line_color="white",
                         marker_line_width=[2, 0],
-                        line_color="#AD0B05" if row["Team"] == "Home" else "#0570B0",
+                        line_color="#AD0B05"
+                        if row["Team"] == "Home"
+                        else "#0570B0",
                         line_width=1,
                         textfont=dict(size=11, color="white"),
                         showlegend=False,
@@ -404,10 +523,14 @@ class EventData:
                         mode="lines+markers+text" if trace else "markers+text",
                         marker_size=[18, 0],
                         marker_symbol="pentagon",
-                        marker_color="#AD0B05" if row["Team"] == "Home" else "#0570B0",
+                        marker_color="#AD0B05"
+                        if row["Team"] == "Home"
+                        else "#0570B0",
                         marker_line_color="white",
                         marker_line_width=[2, 0],
-                        line_color="#AD0B05" if row["Team"] == "Home" else "#0570B0",
+                        line_color="#AD0B05"
+                        if row["Team"] == "Home"
+                        else "#0570B0",
                         line_width=1,
                         textfont=dict(size=11, color="white"),
                         showlegend=False,
@@ -427,10 +550,14 @@ class EventData:
                         mode="lines+markers+text" if trace else "markers+text",
                         marker_size=[18, 0],
                         marker_symbol="pentagon",
-                        marker_color="#AD0B05" if row["Team"] == "Home" else "#0570B0",
+                        marker_color="#AD0B05"
+                        if row["Team"] == "Home"
+                        else "#0570B0",
                         marker_line_color="white",
                         marker_line_width=[2, 0],
-                        line_color="#AD0B05" if row["Team"] == "Home" else "#0570B0",
+                        line_color="#AD0B05"
+                        if row["Team"] == "Home"
+                        else "#0570B0",
                         line_width=1,
                         textfont=dict(size=11, color="white"),
                         showlegend=False,
@@ -450,10 +577,14 @@ class EventData:
                         mode="lines+markers+text" if trace else "markers+text",
                         marker_size=[0, 10],
                         marker_symbol="square",
-                        marker_color="#AD0B05" if row["Team"] == "Home" else "#0570B0",
+                        marker_color="#AD0B05"
+                        if row["Team"] == "Home"
+                        else "#0570B0",
                         marker_line_color="white",
                         marker_line_width=[0, 2],
-                        line_color="#AD0B05" if row["Team"] == "Home" else "#0570B0",
+                        line_color="#AD0B05"
+                        if row["Team"] == "Home"
+                        else "#0570B0",
                         line_width=1,
                         textfont=dict(size=11, color="white"),
                         showlegend=False,
@@ -473,10 +604,14 @@ class EventData:
                         mode="markers+text",
                         marker_size=[18, 0],
                         marker_symbol="hexagon",
-                        marker_color="#AD0B05" if row["Team"] == "Home" else "#0570B0",
+                        marker_color="#AD0B05"
+                        if row["Team"] == "Home"
+                        else "#0570B0",
                         marker_line_color="white",
                         marker_line_width=[1, 0],
-                        line_color="#AD0B05" if row["Team"] == "Home" else "#0570B0",
+                        line_color="#AD0B05"
+                        if row["Team"] == "Home"
+                        else "#0570B0",
                         line_width=1,
                         textfont=dict(size=11, color="white"),
                         showlegend=False,
@@ -496,10 +631,14 @@ class EventData:
                         mode="markers+text",
                         marker_size=[18, 0],
                         marker_symbol="hexagon",
-                        marker_color="#AD0B05" if row["Team"] == "Home" else "#0570B0",
+                        marker_color="#AD0B05"
+                        if row["Team"] == "Home"
+                        else "#0570B0",
                         marker_line_color="white",
                         marker_line_width=[1, 0],
-                        line_color="#AD0B05" if row["Team"] == "Home" else "#0570B0",
+                        line_color="#AD0B05"
+                        if row["Team"] == "Home"
+                        else "#0570B0",
                         line_width=1,
                         textfont=dict(size=11, color="white"),
                         showlegend=False,
@@ -522,7 +661,9 @@ class EventData:
                         # marker_color=row["marker_color"],
                         # marker_line_color=row["marker_line_color"],
                         # marker_line_width=[row["marker_line_width"], 0],
-                        line_color="#AD0B05" if row["Team"] == "Home" else "#0570B0",
+                        line_color="#AD0B05"
+                        if row["Team"] == "Home"
+                        else "#0570B0",
                         line_width=1,
                         # textfont=dict(size=11, color="white"),
                         showlegend=False,
@@ -542,7 +683,9 @@ class EventData:
                         mode="markers+text",
                         marker_size=[12, 0],
                         marker_symbol="diamond",
-                        marker_color="#AD0B05" if row["Team"] == "Home" else "#0570B0",
+                        marker_color="#AD0B05"
+                        if row["Team"] == "Home"
+                        else "#0570B0",
                         marker_line_color="white",
                         marker_line_width=[1, 0],
                         # line_color="#AD0B05" if row["Team"] == "Home" else "#0570B0",
@@ -565,7 +708,9 @@ class EventData:
                         mode="markers+text",
                         marker_size=[0, 12],
                         marker_symbol="diamond",
-                        marker_color="#AD0B05" if row["Team"] == "Home" else "#0570B0",
+                        marker_color="#AD0B05"
+                        if row["Team"] == "Home"
+                        else "#0570B0",
                         marker_line_color="white",
                         marker_line_width=[0, 1],
                         # line_color="#AD0B05" if row["Team"] == "Home" else "#0570B0",
